------------------------------------------------------------------------------------------------------
-- Abandon Ship
------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------
-- GLOBAL CONSTANTS
------------------------------------------------------------------------------------------------------
EB_DEBUG = 0;
EB_DEBUG_WINDOW = 0;

EB_START_MOVING = 0;

EB_DIR_X = 0;
EB_DIR_Y = 0;
EB_DIR_SIZE = 0;

EB_OLDPOS_X = -1;
EB_OLDPOS_Y = -1;
EB_POS_X = -1;
EB_POS_Y = -1;

EB_STUCK_COUNTER = nil;
EB_STUCK_POS_X = 0;
EB_STUCK_POS_Y = 0;
EB_DIR_TOGGLES = 0;
EB_STUCK = 0;

EB_CIRCLING_TOLERANCE_TIME = 3;
EB_CIRCLING_TOLERANCE = 0.001;
EB_CIRCLING_TIME = nil;

EB_CURRENT_ZONE = "*";
EB_PREVIOUS_ZONE = "";
EB_GET_SUBZONE_TRIES = 0;

EB_SHOW_NOTIFY = 0;

EB_START_TIMER = 0;
EB_TIMER_COUNTDOWN = 30;
EB_TIMER_COUNTER = 0;

EB_MAX_POINTS = 3;
EB_CURRENT_POINT = -1;

EB_ROUTE_INDEX = 0;
EB_ROUTE_INDEX_ALT = 0;

EB_ARRIVAL_TOLERANCE = 0.0001;

EB_DEST_DELAY = 0;
EB_ROUTE_NAMES = {};
EB_PATH_X = {};
EB_PATH_Y = {};
EB_PATHS_X = {};
EB_PATHS_Y = {};

EB_IGNORED_ZONES = {};
EB_IGNORED_ZONES_ALT = {};

EB_USE_INFINITE_LINES = 0;

EB_DEST_DELAY_ALT = 0;
EB_ROUTE_NAMES_ALT = {};
EB_ROUTE_MULTIPLE_ROUTES = 0;

EB_EDGE_X = {};
EB_EDGE_Y = {};
EB_LAST_EDGE_DISTANCE = 0;
EB_CHECK_EDGE_DISTANCE = 0;
EB_EDGE_TOLERANCE = 0.005;

EB_RUN = 0;
EB_WALK = 1;
EB_RUN_WALK = EB_RUN;

EB_TRAM_MODE = 0;
EB_TRAM_STORMWIND = 2;
EB_TRAM_IRONFORGE = 1;
EB_TRAM_BEGIN = 0;
EB_TRAM_END = 0;
EB_TRAM_MOVE_TIME = 2500;
EB_SAVED_TRAM_VAL = -1;

EB_START_FI_TIMER = 0;
EB_FI_TIMER_COUNTER = 0;
EB_FI_TIMER_COUNTDOWN = 1;

EB_SAVE_PREFIX = "";

EB_SET_ZONE_RET = false;

EB_IGNORED_ZONE_HIT = 0;
EB_IGNORED_ZONE_HIT_ALT = 0;

EB_Old_UIErrorsFrame_OnEvent = nil;
------------------------------------------------------------------------------------------------------
-- UTILITY FUNCTIONS
------------------------------------------------------------------------------------------------------
local function dout(msg)
	if( DEFAULT_CHAT_FRAME and EB_DEBUG == 1) then
		DEFAULT_CHAT_FRAME:AddMessage(msg,1.0,0,0);
	end
end

function EB_SetTramMode()
	EB_TRAM_MODE = 1;
	-- Hook function
	if (UIErrorsFrame_OnEvent ~= EB_New_UIErrorsFrame_OnEvent) then
		EB_Old_UIErrorsFrame_OnEvent = UIErrorsFrame_OnEvent;
		UIErrorsFrame_OnEvent = EB_New_UIErrorsFrame_OnEvent;
		--dout("Set UIErrors to new");
	end

	--dout("Set tram mode!");
end

function EB_DoTramTimer(info)
	EBPortString:SetText(info);
	EBTimerString:SetText(EB_GetRedText(EB_TRAM_WARNING));
	EB_NotifyFrame:Show();
	EB_TIMER_COUNTDOWN = 3;
	EB_TIMER_COUNTER = 0;
end

function EB_DoTimer(portName,delay)
	if (AbandonShipOptions.boatsAndZepplins == 1) then
		EBPortString:SetText(EB_NOW_ARRIVING_IN_1.."\""..portName.."\""..EB_NOW_ARRIVING_IN_2);
		EB_NotifyFrame:Show();
		EB_TIMER_COUNTDOWN = delay;
		EB_TIMER_COUNTER = 0;
		EB_START_TIMER = 1;
		EB_ROUTE_INDEX = 0;
		EB_ROUTE_INDEX_ALT = 0;
	else
		EB_Reset();
	end
end


function EB_Reset()
	EB_START_MOVING = 0;
	EB_DIR_X = 0;
	EB_DIR_Y = 0;
	EB_DIR_SIZE = 0;
	EB_OLDPOS_X = -1;
	EB_OLDPOS_Y = -1;
	EB_POS_X = -1;
	EB_POS_Y = -1;
	EB_STUCK_COUNTER = nil;
	EB_STUCK_POS_X = 0;
	EB_STUCK_POS_Y = 0;
	EB_STUCK = 0;
	EB_DIR_TOGGLES = 0;
	EB_ROUTE_INDEX = 0;
	EB_ROUTE_INDEX_ALT = 0;
	EB_SHOW_NOTIFY = 0;
	EB_START_TIMER = 0;
	EB_TIMER_COUNTER = 0;
	EB_CURRENT_POINT = -1;
	EB_CIRCLING_TIME = nil;
	EB_GET_SUBZONE_TRIES = 0;
	EB_CHECK_EDGE_DISTANCE = 0;
	EB_LAST_EDGE_DISTANCE = 0;
	EB_TRAM_BEGIN = 0;
	EB_TRAM_END = 0;
	EB_START_FI_TIMER = 0;
	EB_FI_TIMER_COUNTER = 0;
	EB_IGNORED_ZONE_HIT = 0;
	EB_SetZoneArrays();
end


function EB_UnscrewMap()
	SetMapToCurrentZone();
	
	local continent,zone,name
	local zoneID={};
		
	for continent in ipairs{GetMapContinents()} do
		for zone,name in ipairs{GetMapZones(continent)} do
			zoneID[name] = zone;
		end
	end
	
	SetMapZoom(GetCurrentMapContinent(),zoneID[GetRealZoneText()]);
end


function EB_SetDefaultArrays()
	EB_ROUTE_NAMES = EB_ROUTE_DEFAULT;
	EB_ROUTE_NAMES_ALT = EB_ROUTE_DEFAULT;
	EB_IGNORED_ZONES = EB_ROUTE_DEFAULT_IGNORE;
	EB_IGNORED_ZONES_ALT = EB_ROUTE_DEFAULT_IGNORE;
	EB_PATH_X = EB_DEST_DEFAULT_X;
	EB_PATH_Y = EB_DEST_DEFAULT_Y;
	EB_EDGE_X = EB_EDGE_DEFAULT_X;
	EB_EDGE_Y = EB_EDGE_DEFAULT_Y;
	EB_DEST_DELAY = 0;
	--dout("Set Arrays: Default Arrays");
end


function EB_New_UIErrorsFrame_OnEvent(event, message, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9)
	local text = EB_UNKNOWN_UNIT;
	if (text and message) then 
		if (message == text) then 
			return; 
		end 
	end
	
	-- Chain back
	if (EB_Old_UIErrorsFrame_OnEvent ~= nil) then
		EB_Old_UIErrorsFrame_OnEvent(event, message, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9);
	end
end


function EB_GetRedText(text)
	if (text) then
		return RED_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE;
	end
end


function EB_UseFinishingItem() -- Probably for mounts
	if (AbandonShipOptions.useItem == 1) then
		local bag,slot = EB_FindItem(AbandonShipOptions.itemName);
		UseContainerItem(bag,slot);
	end
end


------------------------------------------------------------------------------------------------------
-- MOVEMENT FUNCTIONS
------------------------------------------------------------------------------------------------------
function EB_SetZoneInfo()
	local sZone = GetSubZoneText();
		
	if (sZone == "" or sZone == nil) then -- check the zone
		sZone = GetZoneText();
	end

	if (sZone == EB_CURRENT_ZONE) then
		return false;
	end

	
	if (sZone == EB_DEEPRUN_TRAM and EB_TRAM_MODE == 0) then
		EB_SetTramMode();
		EB_CURRENT_ZONE = sZone;
		return false;
	elseif (sZone ~= EB_DEEPRUN_TRAM and EB_TRAM_MODE == 1) then
		EB_NotifyFrame:Hide();
		EB_TRAM_MODE = 0;
		EB_TRAM_BEGIN = 0;
		EB_TRAM_END = 0;
	else
		EB_TRAM_MODE = 0;
	end

	if (EB_Old_UIErrorsFrame_OnEvent ~= nil and EB_Old_UIErrorsFrame_OnEvent ~= EB_New_UIErrorsFrame_OnEvent) then
		UIErrorsFrame_OnEvent = EB_Old_UIErrorsFrame_OnEvent;
		--dout("Reset UIErrors to old");
	end

	--dout("** "..sZone.." **");
	
	for i=0,(table.getn(EB_IGNORED_ZONES)),1 do
		if (sZone == EB_IGNORED_ZONES[i]) then
			--dout("Ignored Zone: "..sZone);
			EB_IGNORED_ZONE_HIT = 1;
			return false;
		end
	end

	if (EB_ROUTE_MULTIPLE_ROUTES) then
		for i=0,(table.getn(EB_IGNORED_ZONES_ALT)),1 do
			if (sZone == EB_IGNORED_ZONES_ALT[i]) then
				--dout("Ignored Zone(Alt): "..sZone);
				EB_IGNORED_ZONE_HIT_ALT = 1;
				return false;
			end
		end
	end

	EB_PREVIOUS_ZONE = EB_CURRENT_ZONE;
	EB_CURRENT_ZONE = sZone;
	return true;
end


function EB_SetZoneArrays()
	EB_ROUTE_MULTIPLE_ROUTES = 0;
	EB_USE_INFINITE_LINES = 0;
	EB_IGNORED_ZONE_HIT_ALT = 0;
	EB_IGNORED_ZONE_HIT = 0;
	
	if (EB_CURRENT_ZONE == EB_ZONE_BOOTYBAY) then
		EB_ROUTE_NAMES = EB_BOOTYBAY_RATCHET;

		EB_IGNORED_ZONES = EB_BOOTYBAY_RATCHET_IGNORED;

		EB_PATHS_X = EB_PATHS_BOOTYBAY_X;
		EB_PATHS_Y = EB_PATHS_BOOTYBAY_Y;
		EB_EDGE_X = EB_EDGE_BOOTYBAY_X;
		EB_EDGE_Y = EB_EDGE_BOOTYBAY_Y;
		EB_DEST_DELAY = EB_DEST_RATCHET_DELAY;
		--dout("Set Arrays: Booty Bay to Ratchet");
	elseif (EB_CURRENT_ZONE == EB_ZONE_RATCHET) then
		EB_ROUTE_NAMES = EB_RATCHET_BOOTYBAY;
		EB_PATHS_X = EB_PATHS_RATCHET_X;
		EB_PATHS_Y = EB_PATHS_RATCHET_Y;

		EB_IGNORED_ZONES = EB_RATCHET_BOOTYBAY_IGNORED;

		EB_EDGE_X = EB_EDGE_RATCHET_X;
		EB_EDGE_Y = EB_EDGE_RATCHET_Y;
		EB_DEST_DELAY = EB_DEST_BOOTYBAY_DELAY;
		--dout("Set Arrays: Ratchet to Booty Bay");
	elseif (EB_CURRENT_ZONE == EB_ZONE_BARADINBAY) then -- **
		-- 2 boats leave from here, so figure out which we're on
		EB_UnscrewMap();
		local curPosx,curPosy = GetPlayerMapPosition("player");

		local distanceTheLongWashBoat = 1;
		local distanceTheramoreIsle = 0;
		
		if (curPosx ~= 0 or curPosy ~= 0) then -- not off the map (baradin bay to theramore isle boat)
			distanceTheLongWashBoat = dist(difference(curPosx,curPosy,EB_DEST_BARADINBAY1_X[0],EB_DEST_BARADINBAY1_Y[0]));
			distanceTheramoreIsle = dist(difference(curPosx,curPosy,EB_DEST_BARADINBAY2_X[0],EB_DEST_BARADINBAY2_Y[0]));
		end

		--dout(distanceTheLongWashBoat.." < "..distanceTheramoreIsle);

		if (distanceTheLongWashBoat < distanceTheramoreIsle) then -- taking long wash boat
			EB_ROUTE_NAMES = EB_BARADINBAY_THELONGWASH;
			EB_PATHS_X = EB_PATHS_BARADINBAY1_X;
			EB_PATHS_Y = EB_PATHS_BARADINBAY1_Y;

			EB_IGNORED_ZONES = EB_BARADINBAY_THELONGWASH_IGNORED;

			EB_EDGE_X = EB_EDGE_BARADINBAY1_X;
			EB_EDGE_Y = EB_EDGE_BARADINBAY1_Y;
			EB_DEST_DELAY = EB_DEST_THELONGWASH_DELAY;
			--dout("Set Arrays: Baradin Bay to The Long Wash");
		else													-- taking theramore isle boat
			EB_ROUTE_NAMES = EB_BARADINBAY_THERAMOREISLE;
			EB_PATHS_X = EB_PATHS_BARADINBAY2_X;
			EB_PATHS_Y = EB_PATHS_BARADINBAY2_Y;

			EB_IGNORED_ZONES = EB_BARADINBAY_THERAMOREISLE_IGNORED;

			EB_EDGE_X = EB_EDGE_BARADINBAY2_X;
			EB_EDGE_Y = EB_EDGE_BARADINBAY2_Y;
			EB_DEST_DELAY = EB_DEST_THERAMOREISLE_DELAY;
			--dout("Set Arrays: Baradin Bay to Theramore Isle");
		end
	elseif (EB_CURRENT_ZONE == EB_ZONE_THELONGWASH) then
		EB_ROUTE_NAMES = EB_THELONGWASH_BARADINBAY;
		EB_PATHS_X = EB_PATHS_THELONGWASH_X;
		EB_PATHS_Y = EB_PATHS_THELONGWASH_Y;

		EB_IGNORED_ZONES = EB_THELONGWASH_BARADINBAY_IGNORED;

		EB_EDGE_X = EB_EDGE_THELONGWASH_X;
		EB_EDGE_Y = EB_EDGE_THELONGWASH_Y;
		EB_DEST_DELAY = EB_DEST_BARADINBAY_DELAY1;
		--dout("Set Arrays: The Long Wash to Baradin Bay");
	elseif (EB_CURRENT_ZONE == EB_ZONE_THERAMOREISLE) then
		EB_ROUTE_NAMES = EB_THERAMOREISLE_BARADINBAY;
		EB_PATHS_X = EB_PATHS_THERAMOREISLE_X;
		EB_PATHS_Y = EB_PATHS_THERAMOREISLE_Y;

		EB_IGNORED_ZONES = EB_THERAMOREISLE_BARADINBAY_IGNORED;

		EB_EDGE_X = EB_EDGE_THERAMOREISLE_X;
		EB_EDGE_Y = EB_EDGE_THERAMOREISLE_Y;
		EB_DEST_DELAY = EB_DEST_BARADINBAY_DELAY2;
		--dout("Set Arrays: Theramore Isle to Baradin Bay");
	elseif (EB_CURRENT_ZONE == EB_ZONE_MISTSEDGE) then
		EB_ROUTE_NAMES = EB_MISTSEDGE_RUTTHERANVILLAGE;
		EB_PATHS_X = EB_PATHS_MISTSEDGE_X;
		EB_PATHS_Y = EB_PATHS_MISTSEDGE_Y;

		EB_IGNORED_ZONES = EB_MISTSEDGE_RUTTHERANVILLAGE_IGNORED;

		EB_EDGE_X = EB_EDGE_MISTSEDGE_X;
		EB_EDGE_Y = EB_EDGE_MISTSEDGE_Y;
		EB_DEST_DELAY = EB_DEST_RUTTHERANVILLAGE_DELAY;
		--dout("Set Arrays: Mist's Edge to Rut'theran Village");
	elseif (EB_CURRENT_ZONE == EB_ZONE_RUTTHERANVILLAGE) then
		EB_ROUTE_NAMES = EB_RUTTHERANVILLAGE_MISTSEDGE;
		EB_PATHS_X = EB_PATHS_RUTTHERANVILLAGE_X;
		EB_PATHS_Y = EB_PATHS_RUTTHERANVILLAGE_Y;

		EB_IGNORED_ZONES = EB_RUTTHERANVILLAGE_MISTSEDGE_IGNORED;

		EB_EDGE_X = EB_EDGE_RUTTHERANVILLAGE_X;
		EB_EDGE_Y = EB_EDGE_RUTTHERANVILLAGE_Y;
		EB_DEST_DELAY = EB_DEST_MISTSEDGE_DELAY;
		--dout("Set Arrays: Rut'theran Village to Mist's Edge");
	elseif (EB_CURRENT_ZONE == EB_ZONE_TIRISFALGLADES) then
		EB_USE_INFINITE_LINES = 1;
		EB_ROUTE_MULTIPLE_ROUTES = 1;
		EB_ROUTE_NAMES = EB_TIRISFALGLADES_DUROTAR;
		EB_ROUTE_NAMES_ALT = EB_TIRISFALGLADES_GROMGOL;
		EB_DEST_DELAY = EB_DEST_DUROTAR_DELAY1;
		EB_DEST_DELAY_ALT = EB_DEST_GROMGOL_DELAY1;

		EB_IGNORED_ZONES = EB_TIRISFALGLADES_DUROTAR_IGNORED;
		EB_IGNORED_ZONES_ALT = EB_TIRISFALGLADES_GROMGOL_IGNORED;

		EB_UnscrewMap();
		local curPosx,curPosy = GetPlayerMapPosition("player");

		local distanceDurotarBoat = 1;
		local distanceGromgolBoat = 0;
		
		distanceDurotarBoat = dist(difference(curPosx,curPosy,EB_DEST_TIRISFALGLADES1_1_X[0],EB_DEST_TIRISFALGLADES1_1_Y[0]));
		distanceGromgolBoat = dist(difference(curPosx,curPosy,EB_DEST_TIRISFALGLADES2_1_X[0],EB_DEST_TIRISFALGLADES2_1_Y[0]));
				
		if (distanceDurotarBoat < distanceGromgolBoat) then
			EB_PATHS_X = EB_PATHS_TIRISFALGLADES1_X;
			EB_PATHS_Y = EB_PATHS_TIRISFALGLADES1_Y;
			EB_EDGE_X = EB_EDGE_TIRISFALGLADES1_X;
			EB_EDGE_Y = EB_EDGE_TIRISFALGLADES1_Y;
			--dout("Set Arrays: Tirisfal Glades (from Durotar) to Durotar or Grom'gol");	
		else
			EB_PATHS_X = EB_PATHS_TIRISFALGLADES2_X;
			EB_PATHS_Y = EB_PATHS_TIRISFALGLADES2_Y;
			EB_EDGE_X = EB_EDGE_TIRISFALGLADES2_X;
			EB_EDGE_Y = EB_EDGE_TIRISFALGLADES2_Y;
			--dout("Set Arrays: Tirisfal Glades (from Grom'gol) to Durotar or Grom'gol");	
		end
	elseif (EB_CURRENT_ZONE == EB_ZONE_DUROTAR) then
		EB_USE_INFINITE_LINES = 1;
		EB_ROUTE_MULTIPLE_ROUTES = 1;
		EB_ROUTE_NAMES = EB_DUROTAR_TIRISFALGLADES;
		EB_ROUTE_NAMES_ALT = EB_DUROTAR_GROMGOL;
		EB_DEST_DELAY = EB_DEST_TIRISFALGLADES_DELAY1;
		EB_DEST_DELAY_ALT = EB_DEST_GROMGOL_DELAY2;

		EB_IGNORED_ZONES = EB_DUROTAR_TIRISFALGLADES_IGNORED;
		EB_IGNORED_ZONES_ALT = EB_DUROTAR_GROMGOL_IGNORED;

		EB_UnscrewMap();
		local curPosx,curPosy = GetPlayerMapPosition("player");

		local distanceTirisfalBoat = 1;
		local distanceGromgalBoat = 0;
		
		distanceTirisfalBoat = dist(difference(curPosx,curPosy,EB_DEST_DUROTAR1_1_X[0],EB_DEST_DUROTAR1_1_Y[0]));
		distanceGromgolBoat = dist(difference(curPosx,curPosy,EB_DEST_DUROTAR2_1_X[0],EB_DEST_DUROTAR2_1_Y[0]));

		if (distanceTirisfalBoat < distanceGromgolBoat) then -- coming from Tirisfal Glades
			EB_PATHS_X = EB_PATHS_DUROTAR1_X;
			EB_PATHS_Y = EB_PATHS_DUROTAR1_Y;
			EB_EDGE_X = EB_EDGE_DUROTAR1_X;
			EB_EDGE_Y = EB_EDGE_DUROTAR1_Y;
			--dout("Set Arrays: Durotar (from Tirisfal Glades) to Tirisfal Glades or Grom'gol");	
		else
			EB_PATHS_X = EB_PATHS_DUROTAR2_X;
			EB_PATHS_Y = EB_PATHS_DUROTAR2_Y;
			EB_EDGE_X = EB_EDGE_DUROTAR2_X;
			EB_EDGE_Y = EB_EDGE_DUROTAR2_Y;
			--dout("Set Arrays: Durotar (from Grom'gol) to Tirisfal Glades or Grom'gol");
		end
	elseif (EB_CURRENT_ZONE == EZ_ZONE_GROMGOLBASECAMP) then
		EB_USE_INFINITE_LINES = 1;
		EB_ROUTE_MULTIPLE_ROUTES = 1;
		EB_ROUTE_NAMES = EB_GROMGOL_TIRISFALGLADES;
		EB_ROUTE_NAMES_ALT = EB_GROMGOL_DUROTAR;
		EB_DEST_DELAY = EB_DEST_TIRISFALGLADES_DELAY2;
		EB_DEST_DELAY_ALT = EB_DEST_DUROTAR_DELAY2;

		EB_IGNORED_ZONES = EB_GROMGOL_TIRISFALGLADES_IGNORED;
		EB_IGNORED_ZONES_ALT = EB_GROMGOL_DUROTAR_IGNORED;

		EB_UnscrewMap();
		local curPosx,curPosy = GetPlayerMapPosition("player");

		local distanceTirisfalBoat = 1;
		local distanceDurotarBoat = 0;
		
		distanceTirisfalBoat = dist(difference(curPosx,curPosy,EB_DEST_GROMGOL1_1_X[0],EB_DEST_GROMGOL1_1_Y[0]));
		distanceDurotarBoat = dist(difference(curPosx,curPosy,EB_DEST_GROMGOL2_1_X[0],EB_DEST_GROMGOL2_1_Y[0]));
				
		if (distanceTirisfalBoat < distanceDurotarBoat) then
			EB_PATHS_X = EB_PATHS_GROMGOL1_X;
			EB_PATHS_Y = EB_PATHS_GROMGOL1_Y;
			EB_EDGE_X = EB_EDGE_GROMGOL1_X;
			EB_EDGE_Y = EB_EDGE_GROMGOL1_Y;
			--dout("Set Arrays: Grom'gol (from Tirisfal Glades) to Tirisfal Glades or Durotar");	
		else
			EB_PATHS_X = EB_PATHS_GROMGOL2_X;
			EB_PATHS_Y = EB_PATHS_GROMGOL2_Y;
			EB_EDGE_X = EB_EDGE_GROMGOL2_X;
			EB_EDGE_Y = EB_EDGE_GROMGOL2_Y;
			--dout("Set Arrays: Grom'gol (from Durotar) to Tirisfal Glades or Durotar");
		end
	elseif (EB_CURRENT_ZONE == EB_ZONE_THEFORGOTTENCOAST) then
		EB_ROUTE_NAMES = EB_THEFORGOTTENCOAST_FEATHERMOONSTRONGHOLD;
		EB_DEST_DELAY = EB_DEST_FEATHERMOONSTRONGHOLD_DELAY;

		EB_IGNORED_ZONES = EB_THEFORGOTTENCOAST_FEATHERMOONSTRONGHOLD_IGNORED;
		
		EB_PATHS_X = EB_PATHS_THEFORGOTTENCOAST_X;
		EB_PATHS_Y = EB_PATHS_THEFORGOTTENCOAST_Y;
		EB_EDGE_X = EB_EDGE_THEFORGOTTENCOAST_X;
		EB_EDGE_Y = EB_EDGE_THEFORGOTTENCOAST_Y;
		--dout("Set Arrays: The Forgotten Coast to Feathermoon Stronghold");
	elseif (EB_CURRENT_ZONE == EB_ZONE_FEATHERMOONSTRONGHOLD) then
		EB_ROUTE_NAMES = EB_FEATHERMOONSTRONGHOLD_THEFORGOTTENCOAST;
		EB_DEST_DELAY = EB_DEST_THEFORGOTTENCOAST_DELAY;

		EB_IGNORED_ZONES = EB_FEATHERMOONSTRONGHOLD_THEFORGOTTENCOAST_IGNORED;
		
		EB_PATHS_X = EB_PATHS_FEATHERMOONSTRONGHOLD_X;
		EB_PATHS_Y = EB_PATHS_FEATHERMOONSTRONGHOLD_Y;
		EB_EDGE_X = EB_EDGE_FEATHERMOONSTRONGHOLD_X;
		EB_EDGE_Y = EB_EDGE_FEATHERMOONSTRONGHOLD_Y;
		--dout("Set Arrays: Feathermoon Stronghold to The Forgotten Coast");
	else
		EB_SetDefaultArrays();
	end
end


-- Return false if not tracking route, true otherwise
function EB_TrackRoute()
	if (UnitOnTaxi("player") == nil) then
		--dout("if ("..EB_PREVIOUS_ZONE.." == "..EB_ROUTE_NAMES[EB_ROUTE_INDEX].." and "..EB_CURRENT_ZONE.." == "..EB_ROUTE_NAMES[EB_ROUTE_INDEX+1]..")");
		if (EB_PREVIOUS_ZONE == EB_ROUTE_NAMES[EB_ROUTE_INDEX] and EB_CURRENT_ZONE == EB_ROUTE_NAMES[EB_ROUTE_INDEX+1]) then
			--dout("Leaving: "..EB_ROUTE_NAMES[EB_ROUTE_INDEX]);
			--dout("Entering: "..EB_ROUTE_NAMES[EB_ROUTE_INDEX+1].." **Confirmed!");
			
			EB_ROUTE_INDEX = EB_ROUTE_INDEX + 1;

			if (EB_ROUTE_INDEX == table.getn(EB_ROUTE_NAMES) and EB_IGNORED_ZONE_HIT == 1) then -- reached final dest
				-- Show Window and start timer
				EB_DoTimer(EB_ROUTE_NAMES[EB_ROUTE_INDEX],EB_DEST_DELAY);
				EB_IGNORED_ZONE_HIT = 0;
				return false;
			end

			return true;
		else -- nothing so reset
			EB_ROUTE_INDEX = 0;
			return false;
		end
	else
		return false;
	end
end


function EB_TrackRouteAlt()
	if (EB_ROUTE_MULTIPLE_ROUTES == 0) then
		return false;
	end
	
	if (UnitOnTaxi("player") == nil) then
		--dout("if ("..EB_PREVIOUS_ZONE.." == "..EB_ROUTE_NAMES_ALT[EB_ROUTE_INDEX_ALT].." and "..EB_CURRENT_ZONE.." == "..EB_ROUTE_NAMES_ALT[EB_ROUTE_INDEX_ALT+1]..")");
		if (EB_PREVIOUS_ZONE == EB_ROUTE_NAMES_ALT[EB_ROUTE_INDEX_ALT] and EB_CURRENT_ZONE == EB_ROUTE_NAMES_ALT[EB_ROUTE_INDEX_ALT+1]) then
			--dout("Leaving: "..EB_ROUTE_NAMES_ALT[EB_ROUTE_INDEX_ALT]);
			--dout("Entering: "..EB_ROUTE_NAMES_ALT[EB_ROUTE_INDEX_ALT+1].." **Confirmed!");
			
			EB_ROUTE_INDEX_ALT = EB_ROUTE_INDEX_ALT + 1;

			if (EB_ROUTE_INDEX_ALT == table.getn(EB_ROUTE_NAMES_ALT) and EB_IGNORED_ZONE_HIT_ALT == 1) then -- reached final dest
				-- Show Window and start timer
				EB_DoTimer(EB_ROUTE_NAMES_ALT[EB_ROUTE_INDEX_ALT],EB_DEST_DELAY_ALT);
				EB_IGNORED_ZONE_HIT_ALT = 0;
				return false;
			end

			return true;
		else -- nothing so reset
			EB_ROUTE_INDEX_ALT = 0;
			return false;
		end
	else
		return false;
	end
end


function EB_SetNextPoint(zone)
	--dout("Reached point: "..EB_CURRENT_POINT..vector_tostring(EB_DEST_X,EB_DEST_Y));
	EB_CURRENT_POINT = EB_CURRENT_POINT + 1;

	EB_CIRCLING_TIME = nil;
	
	if (EB_CURRENT_POINT > table.getn(EB_PATH_X)) then -- final dest
		return -1;
	end

	EB_DEST_X = EB_PATH_X[EB_CURRENT_POINT];
	EB_DEST_Y = EB_PATH_Y[EB_CURRENT_POINT];

	return 1;
end


function EB_SetStartingPoint()
	local curPosx,curPosy = GetPlayerMapPosition("player");

	local smallestDistance = 10000;
	local closestPointx = 0;
	local closestPointy = 0;
	local closestIndex = 0;
	local pathIndex = 0;
	local tangentPointFunction = tangetpointonline;

	if (EB_USE_INFINITE_LINES == 1) then
		tangentPointFunction = tangetpointoninfiniteline;
	end

	for i=0,table.getn(EB_PATHS_X),1 do
		--dout("path: "..i);
		-- get the tanget point for all path lines
		local pathx = EB_PATHS_X[i];
		local pathy = EB_PATHS_Y[i];
		for j=0,(table.getn(pathx)-2),1 do
			--dout("point: "..j);
			local tangentPointx,tangentPointy = tangentPointFunction(curPosx,curPosy,pathx[j],pathy[j],pathx[j+1],pathy[j+1]);

			if (tangentPointx ~= 0 or tangentPointy ~= 0) then
				local pDistance = dist(difference(curPosx,curPosy,tangentPointx,tangentPointy));

				if (pDistance < smallestDistance) then
					smallestDistance = pDistance;
					closestPointx = tangentPointx;
					closestPointy = tangentPointy;
					closestIndex = j;
					pathIndex = i;
				end
			end
		end
	end

	if (smallestDistance == 10000) then -- no tangent point found
		--dout("No tangent point found");

		for i=0,table.getn(EB_PATHS_X),1 do
			local pathx = EB_PATHS_X[i];
			local pathy = EB_PATHS_Y[i];

			local startDistance = dist(difference(curPosx,curPosy,pathx[0],pathy[0]));
			local endDistance = dist(difference(curPosx,curPosy,pathx[table.getn(pathx)],pathy[table.getn(pathy)]));

			if (startDistance < smallestDistance) then
				smallestDistance = startDistance;
				closestPointx = pathx[0];
				closestPointy = pathy[0];
				closestIndex = 0;
				pathIndex = i;
			end

			if (endDistance < smallestDistance) then
				smallestDistance = endDistance;
				closestPointx = pathx[table.getn(pathx)];
				closestPointy = pathy[table.getn(pathy)];
				closestIndex = table.getn(pathx);
				pathIndex = i;
			end
		end
	end

	EB_DEST_X = closestPointx;
	EB_DEST_Y = closestPointy;
	
	EB_PATH_X = EB_PATHS_X[pathIndex];
	EB_PATH_Y = EB_PATHS_Y[pathIndex];

	-- DEBUG
	--dout("closestPoint: "..vector_tostring(closestPointx,closestPointy));
	--dout("closestIndex: "..closestIndex);
	--dout("pathIndex: "..pathIndex);

	return closestIndex;
end


function EB_StopMoving(useFinishingItem)
	EB_START_MOVING = 0;
	MoveForwardStop(0);
	MoveBackwardStop(0);
	TurnLeftStop(0);
	TurnRightStop(0);
	
	if (EB_RUN_WALK == EB_WALK) then
		EB_RUN_WALK = EB_RUN;
		ToggleRun(0);
	end
	
	EB_NotifyFrame:Hide();

	EB_Reset();

	if (useFinishingItem == true) then
		EB_START_FI_TIMER = 1;
	end
end


function EB_BeginMoving()
	if (EB_TRAM_MODE == 1) then -- do tram exit
		MoveForwardStart(0);
		MoveForwardStop(GetTime()*1000 + EB_TRAM_MOVE_TIME);
		ClearTarget();
		EB_Reset();
	else -- do normal exit
		EB_UnscrewMap();
		ToggleRun(0);
		EB_RUN_WALK = EB_WALK;
		--EB_CURRENT_POINT = EB_GetClosestIndex(EB_CURRENT_ZONE);
		EB_CURRENT_POINT = EB_SetStartingPoint();
		--dout("EB_DEST: "..vector_tostring(EB_DEST_X,EB_DEST_Y));
		EB_START_MOVING = 1;
	end
end


function EB_MoveInDirection(dirx,diry)
	if (EB_STUCK == 0) then
		MoveBackwardStop(0);
		MoveForwardStart(0);		
	else
		MoveForwardStop(0);		
		MoveBackwardStart(0);
	end

	if (EB_STUCK_COUNTER) then
		if (dist(difference(EB_POS_X,EB_POS_Y, EB_STUCK_POS_X,EB_STUCK_POS_Y)) < 0.00003) then
			EB_STUCK_COUNTER = EB_STUCK_COUNTER + 1;

			if (EB_STUCK_COUNTER > 10) then
				--dout("** STUCK");
				if (EB_STUCK == 0) then
					EB_STUCK = 1;
				else
					EB_STUCK = 0;
				end

				EB_DIR_TOGGLES = EB_DIR_TOGGLES + 1;
				EB_STUCK_POS_X = nil;
				EB_STUCK_POS_Y = nil;
				EB_STUCK_COUNTER = nil;
				EB_CIRCLING_TIME = nil;
			end
			return;
		else
			EB_STUCK_COUNTER = nil;
			EB_STUCK_POS_X = nil;
			EB_STUCK_POS_Y = nil;
		end
	elseif (EB_DIR_SIZE < 0.00001) then
		EB_STUCK_POS_X = EB_POS_X;
		EB_STUCK_POS_Y = EB_POS_Y;
		EB_STUCK_COUNTER = 0;
		return;
	end

	local dot = dotprod(EB_DIR_X,EB_DIR_Y, dirx,diry);
	
	if (dot < 0.995) then
		if (dot < 0) then
			--MoveForwardStop(0);
		end
	
		local cross = crossprod(EB_DIR_X,EB_DIR_Y, dirx,diry);
			
		if (cross > 0) then
			TurnLeftStop(0);
			TurnRightStart(0);
		else
			TurnRightStop(0);
			TurnLeftStart(0);
		end
	else
		TurnLeftStop(0);
		TurnRightStop(0);
	end
end


function EB_CirclingPoint()
	local distance = dist(difference(EB_POS_X,EB_POS_Y, EB_DEST_X,EB_DEST_Y));
	if (distance < EB_CIRCLING_TOLERANCE) then
		if (EB_CIRCLING_TIME ~= nil) then
			if (time() - EB_CIRCLING_TIME > EB_CIRCLING_TOLERANCE_TIME) then
				EB_CIRCLING_TIME = nil;
				return true;
			end
		else
			EB_CIRCLING_TIME = time();
		end
	else
		EB_CIRCLING_TIME = nil;
	end

	return false;
end


function EB_CheckNearEdge()
	local distanceToEdge = 10000;

	for i=0, table.getn(EB_EDGE_X), 2 do
		local tangentPointx,tangentPointy = tangetpointonline(EB_POS_X,EB_POS_Y,EB_EDGE_X[i],EB_EDGE_Y[i],EB_EDGE_X[i+1],EB_EDGE_Y[i+1]);
		local pDistance = dist(difference(EB_POS_X,EB_POS_Y,tangentPointx,tangentPointy));

		if (tangentPointx ~= 0 or tangentPointy ~= 0) then
			if (pDistance < distanceToEdge) then
				distanceToEdge = pDistance;
			end
		end
	end
	
	if (distanceToEdge < 0.0002) then
		----dout("Checking Edge...");
		if (EB_CHECK_EDGE_DISTANCE == 0) then -- monitoring distance from edge for comparison
			EB_CHECK_EDGE_DISTANCE = 1;
			EB_LAST_EDGE_DISTANCE = distanceToEdge;
		else
			if (distanceToEdge < EB_LAST_EDGE_DISTANCE) then -- moving toward edge
				--dout("** Moving toward edge, reversing direction");
				if (EB_STUCK == 0) then
					EB_STUCK = 1;
				else
					EB_STUCK = 0;
				end
			end

			EB_CHECK_EDGE_DISTANCE = 0;
		end
	else
		EB_CHECK_EDGE_DISTANCE = 0;
	end
end
------------------------------------------------------------------------------------------------------
-- EVENT HANDLERS
------------------------------------------------------------------------------------------------------
function EB_OnLoad()
	if (EB_DEBUG_WINDOW == 1) then
		EB_DebugFrame:Show();
	else
		EB_DebugFrame:Hide();
	end
	
	this:RegisterEvent("ZONE_CHANGED");
	this:RegisterEvent("ZONE_CHANGED_INDOORS");
	this:RegisterEvent("ZONE_CHANGED_NEW_AREA");
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_REGEN_DISABLED");
	--this:RegisterEvent("PLAYER_LEAVE_COMBAT");
	
	EB_SetDefaultArrays();

	SLASH_ABANDONSHIP1 = "/abandonship";
	SLASH_ABANDONSHIP2 = "/as";
	SlashCmdList["ABANDONSHIP"] = function(msg)
		EB_SlashCommandHandler(msg);
	end

	AbandonShipOptions = { };
	AbandonShipOptions.itemName = "";
	--AbandonShipOptions.bag = -1;
	--AbandonShipOptions.slot = -1;
	AbandonShipOptions.useItem = 0;
	AbandonShipOptions.boatsAndZepplins = 1;
	AbandonShipOptions.tram = 1;
	AbandonShipOptions.attacked = 1;

	EB_DoLocalization();
end


function EB_SlashCommandHandler(msg)
	if (msg == "" or msg == nil) then
		EBOptionsFrame:Show();
	end
end


function EB_OnEvent(arg1) -- 
	if (arg1 == "ZONE_CHANGED") or (arg1 == "ZONE_CHANGED_INDOORS") or (arg1 == "ZONE_CHANGED_NEW_AREA") then
		if (EB_SetZoneInfo() == true) then -- have new zone info
			EB_TRACK_ROUTE = EB_TrackRoute(); 
			EB_TRACK_ROUTE_ALT = EB_TrackRouteAlt();
			--trackRouteRet = EB_TrackRoute();
			--local trackRouteAltRet = EB_TrackRouteAlt();
			
			if (EB_TRACK_ROUTE == false and EB_TRACK_ROUTE_ALT == false) then -- not tracking route, so reset array info at will
				EB_SetZoneArrays();
			end
		end
	elseif (arg1 == "PLAYER_REGEN_DISABLED") then
		--dout("Entered combat.");
		-- Check to see if we need to tuen anything off
		if (AbandonShipOptions.attacked == 1 and (EB_START_TIMER == 1 or EB_TRAM_MODE ==1)) then
			--dout("Entered combat... Resetting");
			EB_StopMoving();
		end
	elseif (arg1 == "PLAYER_LEAVE_COMBAT") then
		--dout("Leaving combat.");
	elseif (arg1 == "PLAYER_ENTERING_WORLD") then
		EB_SAVE_PREFIX = UnitName("player");
		EB_SAVE_PREFIX = EB_SAVE_PREFIX..GetCVar("realmName");
		
		if (AbandonShipOptions == nil) then
			AbandonShipOptions = { };
			AbandonShipOptions.itemName = "";
			AbandonShipOptions.useItem = 0;
			AbandonShipOptions.boatsAndZepplins = 1;
			AbandonShipOptions.tram = 1;
			AbandonShipOptions.attacked = 1;
		end

		AbandonShipOptions.itemName = AbandonShipOptions[EB_SAVE_PREFIX.."itemName"];

		if (AbandonShipOptions.itemName == nil) then
			AbandonShipOptions.itemName = "";
		end

		if (AbandonShipOptions.tram < 0) then
			--dout("Resetting tram?");
			AbandonShipOptions.tram = 1;
		end

		if (AbandonShipOptions.useItem == nil or AbandonShipOptions.useItem  < 0) then
			--dout("Resetting useItem?");
			AbandonShipOptions.useItem = 0;
		end 

		if (AbandonShipOptions.boatsAndZepplins == nil or AbandonShipOptions.boatsAndZepplins  < 0) then
			--dout("Resetting boatsAndZepplins?");
			AbandonShipOptions.boatsAndZepplins = 1;
		end 

		if (AbandonShipOptions.attacked == nil or AbandonShipOptions.attacked < 0) then
			--dout("Resetting attacked?");
			AbandonShipOptions.attacked = 1;
		end 

		--dout("Item Name: "..AbandonShipOptions.itemName);
		
		--AbandonShipOptions.bag, AbandonShipOptions.slot = EB_FindItem(AbandonShipOptions.itemName);
		----dout("OnEvent--> bag: "..AbandonShipOptions.bag.." slot: "..AbandonShipOptions.slot);
	end
end


function EB_OnUpdate(elapsed)
	-- Sometimes, GetSubZoneText() returns nothing during onload
	if (EB_CURRENT_ZONE == "*" and EB_GET_SUBZONE_TRIES < 10) then -- not set
		--EB_UnscrewMap();
		if (EB_SetZoneInfo() == true) then 
			EB_SetZoneArrays();
			EB_GET_SUBZONE_TRIES = 0;
		else
			EB_GET_SUBZONE_TRIES = EB_GET_SUBZONE_TRIES + 1;
		end
	end
	
	----dout(EB_TRAM_MODE..", "..AbandonShipOptions.tram);
	if (EB_TRAM_MODE == 1 and AbandonShipOptions.tram == 1) then
		if (UnitName("target") ~= EB_STORMWIND_ARRIVAL and UnitName("target") ~= EB_IRONFORGE_ARRIVAL) then	
			TargetByName(EB_STORMWIND_ARRIVAL);
			
			if (UnitName("target") == EB_STORMWIND_ARRIVAL) then
				if (EB_TRAM_BEGIN == 0 and EB_START_TIMER == 0) then -- no starting point set yet
					--dout("** Starting in Stormwind");
					EB_DoTramTimer(EB_TRAM_STORMWIND_IRONFORGE);
					EB_TRAM_BEGIN = EB_TRAM_STORMWIND;
					EB_TRAM_END = EB_TRAM_IRONFORGE;
				elseif (EB_TRAM_END == EB_TRAM_STORMWIND) then -- This is the destination
					-- start countdown here
					EB_START_TIMER = 1;
					--dout("Starting stormwind exit");
					EB_TRAM_BEGIN = 0;
					EB_TRAM_END = 0;
				end
			end

			if (UnitName("target") ~= EB_STORMWIND_ARRIVAL) then
				TargetByName(EB_IRONFORGE_ARRIVAL);
			end
			
			if (UnitName("target") == EB_IRONFORGE_ARRIVAL) then
				if (EB_TRAM_BEGIN == 0 and EB_START_TIMER == 0) then -- no starting point set yet
					--dout("** Starting in Ironforge");
					EB_DoTramTimer(EB_TRAM_IRONFORGE_STORMWIND);
					EB_TRAM_BEGIN = EB_TRAM_IRONFORGE;
					EB_TRAM_END = EB_TRAM_STORMWIND;
				elseif (EB_TRAM_END == EB_TRAM_IRONFORGE) then -- This is the destination
					-- start countdown here
					EB_START_TIMER = 1;
					--dout("Starting ironforge exit");
					EB_TRAM_BEGIN = 0;
					EB_TRAM_END = 0;
				end
			end
		end
	end

	if (EB_START_TIMER == 1) then
		EB_TIMER_COUNTER = EB_TIMER_COUNTER + elapsed;

		local countdown = EB_TIMER_COUNTDOWN-EB_TIMER_COUNTER;

		EBTimerString:SetText(EB_AUTO_EXITING_IN..ceil(countdown)..EB_SECONDS);

		if (countdown < -1) then
			EB_START_TIMER = 0;
			EB_TIMER_COUNTER = 0;
			
			EB_BeginMoving(EB_TRAM_MODE);
			EBTimerString:SetText(EB_NOW_AUTO_EXITING);
		end
	end

	if (EB_START_FI_TIMER == 1) then
		EB_FI_TIMER_COUNTER = EB_FI_TIMER_COUNTER + elapsed;

		local countdown = EB_FI_TIMER_COUNTDOWN-EB_FI_TIMER_COUNTER;

		if (countdown < -1) then
			EB_START_FI_TIMER = 0;
			EB_FI_TIMER_COUNTER = 0;
			
			EB_UseFinishingItem();
		end
	end
	
	if (EB_START_MOVING == 1) then
		EB_OLDPOS_X = EB_POS_X;
		EB_OLDPOS_Y = EB_POS_Y;
		EB_POS_X, EB_POS_Y = GetPlayerMapPosition("player");

		local distance = dist(difference(EB_DEST_X,EB_DEST_Y,EB_POS_X,EB_POS_Y));
		----dout("distance: "..distance);
		if (distance < EB_ARRIVAL_TOLERANCE) then -- reached a point
			if (EB_SetNextPoint(EB_CURRENT_ZONE) == -1) then -- at final dest
				EB_StopMoving(true);
			end
			return;
		end

		-- Check to see if exit looks hopeless
		if (EB_DIR_TOGGLES > 20) then -- call it quits (STUCK!)
			EB_StopMoving();
			--dout("Abandon Ship exit path seems impossible!");
			return;
		end

		-- Check to see if circling point
		if (EB_CirclingPoint() == true) then
			--dout("Circling... moving to next point!");
			if (EB_SetNextPoint(EB_CURRENT_ZONE) == -1) then -- at final dest
				EB_StopMoving(true);
			end
			return;
		else
			-- Check to see if moving toward the edge
			EB_CheckNearEdge();
		end

		----dout("POS: "..EB_POS_X..", "..EB_POS_Y);
		----dout("OLDPOS: "..EB_OLDPOS_X..", "..EB_OLDPOS_Y);

		if (EB_OLDPOS_X == nil or EB_OLDPOS_X == -1) then
			return;
		end
					
		EB_DIR_SIZE = dist(difference(EB_POS_X,EB_POS_Y, EB_OLDPOS_X,EB_OLDPOS_Y));
		EB_DIR_X,EB_DIR_Y = normalize(difference(EB_POS_X,EB_POS_Y, EB_OLDPOS_X,EB_OLDPOS_Y));
		
		local dirx,diry = normalize(difference(EB_DEST_X,EB_DEST_Y, EB_POS_X,EB_POS_Y));
		EB_MoveInDirection(dirx,diry);
	end
end


------------------------------------------------------------------------------------------------------
-- BUTTON EVENT HANDLERS
------------------------------------------------------------------------------------------------------
function EB_TestOnClick()
	EB_UnscrewMap();

	-- TEMP
	--EB_PATHS_X = EB_PATHS_TEST_X;
	--EB_PATHS_Y = EB_PATHS_TEST_Y;

	--EB_PATH_X = EB_DEST_TEST_X;
	--EB_PATH_Y = EB_DEST_TEST_Y;

	--EB_EDGE_X = EB_EDGE_TEST_X;
	--EB_EDGE_Y = EB_EDGE_TEST_Y;

	EB_NotifyFrame:Show();
	EB_BeginMoving();
end


function EB_CancelOnClick()
	if (EB_TRAM_MODE == 1) then
		EB_ConfirmFrame:Show();
	else
		EB_StopMoving();
	end
end

function EB_FunctionOnClick()
	
end


function EB_XY1OnClick()
	EB_UnscrewMap();
	local x,y = GetPlayerMapPosition("player");
	--dout("x: "..x);
	--dout("y: "..y);
	EB_XYFrame1:SetText(x..","..y);
end

function EB_XY2OnClick()
	EB_UnscrewMap();
	local x,y = GetPlayerMapPosition("player");
	--dout("x: "..x);
	--dout("y: "..y);
	EB_XYFrame2:SetText(x..","..y);
end

function EB_XY3OnClick()
	EB_UnscrewMap();
	local x,y = GetPlayerMapPosition("player");
	--dout("x: "..x);
	--dout("y: "..y);
	EB_XYFrame3:SetText(x..","..y);
end

function EB_YesOnClick()
	EB_TRAM_MODE = 0;
	EB_StopMoving();
	EB_ConfirmFrame:Hide();
end

function EB_NoOnClick()
	EB_ConfirmFrame:Hide();
end